==> ewok-0.4.3.tar.gz: ewok-0.4.3/lib/Ewok/Locker/DBM.pm <==
# tag: ewok dbm locker

package Ewok::Locker::DBM;

use Fcntl;
use AnyDBM_File;
use Time::Local;
use Carp;
use Ewok::Config qw($EWOK_LOCKDIR);
use strict;

sub init
{
    my $self = shift;
    my %arg = @_;
    $self->{timeout} = ($arg{timeout} || 0);     # Timeout in seconds

    my %h;
    tie(%h, 'AnyDBM_File', "$EWOK_LOCKDIR/lock.dbm", O_RDWR|O_CREAT, 0600)
        or croak "tie to lock.dbm failed: $!";
    $self->{lockdb} = \%h;
    return $self;
}

sub new 
{
    my $class = shift;
    my $self = {};
    bless $self, $class;
    return $self->init(@_);
}

sub lock
{
    my $self = shift;
    my ($object, $user, $args) = @_;
    
    # Check for a lock on this object
    my $lock = $self->{lockdb}->{$object};
    my $now = timelocal(localtime);
    if ($lock) {
        my ($lock_user, $lock_ts) = split /\|/, $lock;
        # If locked by someone else, check the timestamp
        if ($lock_user ne $user && 
           ($self->{timeout} == 0 || $now - $lock_ts < $self->{timeout})) {
            # If not timed out, fail
            ${$args->{who}} = $lock_user if ref $args->{who} eq 'SCALAR';
            return undef;
        }
    }

    # (Re)Lock the object
    $self->{lockdb}->{$object} = "$user|$now";
    return 1;
}

sub unlock 
{
    my $self = shift;
    my ($object, $user, $args) = @_;

    # Check for a lock on this object
    my $lock = $self->{lockdb}->{$object};
    if ($lock) {
        my ($lock_user, $lock_ts) = split /\|/, $lock;
        # If locked by someone else, fail
        if ($lock_user ne $user) {
            ${$args->{who}} = $lock_user if ref $args->{who} eq 'SCALAR';
            return undef;
        }

        # Else remove the lock
        delete $self->{lockdb}->{$object};
    }

    return 1;
}

sub DESTROY
{
    my $self = shift;
    untie %{$self->{lockdb}} if ref $self->{lockdb} eq 'HASH';
}

1;

==> ewok-0.5.0.tar.gz: ewok-0.5.0/lib/Ewok/Locker/DBM.pm <==
# tag: ewok dbm locker

package Ewok::Locker::DBM;

use Fcntl;
use AnyDBM_File;
use Time::Local;
use Ewok::Config;
use strict;

sub init
{
    my $self = shift;
    my %arg = @_;
    $self->{timeout} = ($arg{timeout} || 0);     # Timeout in seconds

    my $Config = get_config();
    my %h;
    tie(%h, 'AnyDBM_File', "$Config->{EWOK_LOCKDIR}/lock.dbm", O_RDWR|O_CREAT, 0640)
        or die "tie to lock.dbm failed: $!";
    $self->{lockdb} = \%h;
    return $self;
}

sub new 
{
    my $class = shift;
    my $self = {};
    bless $self, $class;
    return $self->init(@_);
}

sub lock
{
    my $self = shift;
    my ($object, $user, $args) = @_;
    
    # Check for a lock on this object
    my $lock = $self->{lockdb}->{$object};
    my $now = timelocal(localtime);
    if ($lock) {
        my ($lock_user, $lock_ts) = split /\|/, $lock;
        # If locked by someone else, check the timestamp
        if ($lock_user ne $user && 
           ($self->{timeout} == 0 || $now - $lock_ts < $self->{timeout})) {
            # If not timed out, fail
            ${$args->{who}} = $lock_user if ref $args->{who} eq 'SCALAR';
            return undef;
        }
    }

    # (Re)Lock the object
    $self->{lockdb}->{$object} = "$user|$now";
    return 1;
}

sub unlock 
{
    my $self = shift;
    my ($object, $user, $args) = @_;

    # Check for a lock on this object
    my $lock = $self->{lockdb}->{$object};
    if ($lock) {
        my ($lock_user, $lock_ts) = split /\|/, $lock;
        # If locked by someone else, fail
        if ($lock_user ne $user) {
            ${$args->{who}} = $lock_user if ref $args->{who} eq 'SCALAR';
            return undef;
        }

        # Else remove the lock
        delete $self->{lockdb}->{$object};
    }

    return 1;
}

sub DESTROY
{
    my $self = shift;
    untie %{$self->{lockdb}} if ref $self->{lockdb} eq 'HASH';
}

1;
